@@ -35,15 +35,17 @@ use IO::Socket::SSL;
 use LWP::UserAgent;
 use HTTP::Request;
 use MIME::Base64 qw();
+use Net::SolarWinds::Log;
 use JSON qw();
 use URI::Encode qw(uri_encode);
 
-our $VERSION=0.04;
+our $VERSION=0.05;
 
 use base qw(Net::SolarWinds::ConstructorHash Net::SolarWinds::LogMethods Net::SolarWinds::Helper);
 use constant RESULT_CLASS=>'Net::SolarWinds::Result';
 use Net::SolarWinds::Result;
 
+
 =head1 GLOBAL OBJECTS
 
 =over 4
@@ -103,6 +105,7 @@ The following constants are accessable via: Net::SolarWinds::REST->CONSTANT
   DEFAULT_PORT=>17778;
   DEFAULT_PROTO=>'https';
   BASE_URI=>'%s://%s:%i/SolarWinds/InformationService/v3/Json/%s';
+  LOG_CLASS=>'Net::SolarWinds::Log';
 
 =cut
 
@@ -112,6 +115,7 @@ use constant DEFAULT_SERVER=>'SolarWindsServer';
 use constant DEFAULT_PORT=>17778;
 use constant DEFAULT_PROTO=>'https';
 use constant BASE_URI=>'%s://%s:%i/SolarWinds/InformationService/v3/Json/%s';
+use constant LOG_CLASS=>'Net::SolarWinds::Log';
 
 =head2 OO Methods
 
@@ -166,7 +170,7 @@ sub build_request {
   my $content=undef;
   my $headers=[@{$self->{header}}];
   if(defined($data)) {
-    $content=$JSON->encode($data);
+    $content=$self->get_json->encode($data);
     push @{$headers},'Content-Length'=>length($content);
   }
   my $request=HTTP::Request->new($method,$uri,$headers,$content);
@@ -184,12 +188,14 @@ was not a 20x value then the object is false.
 
 sub run_request {
   my ($self,$request)=@_;
-  my $response=$UA->request($request);
-  $self->log_debug($response->as_string);
+
+  $self->log_debug("Sending: ",$request->as_string);
+  my $response=$self->get_ua->request($request);
+  $self->log_debug("Got back",$response->as_string);
   my $content=$response->decoded_content;
   if($response->is_success) {
     if($content=~ /^\s*[\[\{]/s) {
-      my $data=eval {$JSON->decode($content)};
+      my $data=eval {$self->get_json->decode($content)};
       if($@) {
         return $self->RESULT_CLASS->new_false("Code: [".$response->code."] JSON Decode error [$@] Content:  $content",$response);
       } else {
@@ -239,6 +245,12 @@ sub DiscoverInterfaceMap {
   return $self->build_interface_result_map($result);
 }
 
+=item my $result=$self->build_interface_result_map($result);
+
+Internals of DiscoverInterfaceMap
+
+=cut
+
 sub build_interface_result_map {
   my ($self,$result)=@_;
 
@@ -344,6 +356,59 @@ sub NodeInterfaceCustomProperties {
   return $result;
 }
 
+=item * my $query=$self->query_finder(@args);
+
+Returns formatted SWQL query within a given function.  It finds the corrisponding constant via the $self->LOG_CLASS->lookback(2) call back.
+
+=cut
+
+sub query_lookup {
+  my ($self,@args)=@_;
+
+  # use the logging class look back to find our method.
+  my $hash=$self->LOG_CLASS->lookback(2);
+
+  my $method=$hash->{sub};
+  $method=~ s/^.*::([^:]+)/$1/s;
+  $self->log_debug("Starting lookup of query: $method");
+  my $query=$self->get_query($method);
+  $self->log_debug("Finished lookup of query: $method");
+
+  $self->log_debug("Preparing query: $method Args: [",join(',',map { defined($_) ? qq{"$_"} : '""' } @args),']');
+  my $swql=$self->prepare_query($query,@args);
+
+  $self->log_debug("Preparing query: $method Looks like: $swql");
+
+  return $swql;
+}
+
+=item * my $prepared=$self->prepare_query($query,@args);
+
+Just a wrapper for: sprintf $query,@args
+
+=cut
+
+sub prepare_query {
+  my ($self,$query,@args)=@_;
+
+  return sprintf $query,@args;
+}
+
+=item * my $raw_query=$self->get_query($method);
+
+Does an internal method lookup of SWQL_$method and returns the results of the method
+
+=cut
+
+sub get_query {
+  my ($self,$method)=@_;
+
+  my $constant="SWQL_$method";
+  return 'QUERY NOT FOUND' unless $self->can($constant);
+
+  return $self->$constant;
+}
+
 =item * my $result=$self->getInterfacesOnNode($NodeID);
 
 Returns a Net::SolarWinds::Result Object
@@ -355,7 +420,7 @@ when false: returns why it failed
 
 sub getInterfacesOnNode {
   my ($self,$node_id)=@_;
-  my $query=qq{SELECT Caption, InterfaceID, DisplayName, FullName, ifname, interfacetype as ifType FROM Orion.NPM.Interfaces where NodeID=$node_id}; 
+  my $query=$self->query_finder($node_id);
 
   $self->log_info("starting $node_id");
   my $result=$self->Query($query);
@@ -430,29 +495,7 @@ Returns a Net::SolarWinds::Result Object: When true it contains an array ref of
 sub getNodesByIp {
   my ($self,$ip)=@_;
   $self->log_info("starting $ip");
-  my $query=qq{SELECT
-  NodeID,
-  IPAddress,
-  IPAddressGUID,
-  Caption,
-  DynamicIP,
-  EngineID,
-  Status,
-  UnManaged,
-  Allow64BitCounters,
-  ObjectSubType,
-  SysObjectID,
-  MachineType,
-  VendorIcon,
-  SNMPVersion,
-  Community,
-  RediscoveryInterval,
-  PollInterval,
-  StatCollection,
-  Uri,
-  DisplayName
-  FROM Orion.Nodes where IP_Address='$ip'
-  };
+  my $query=$self->query_lookup($ip);
   my $result=$self->Query($query);
   $self->log_info("stopping");
   return $result;
@@ -468,58 +511,22 @@ Returns a Net::SolarWinds::Result Object: When true it contains an array ref of
 sub getNodesByDisplayName {
   my ($self,$ip)=@_;
   $self->log_info("starting $ip");
-  my $query=qq{SELECT
-  NodeID,
-  IPAddress,
-  IPAddressGUID,
-  Caption,
-  DynamicIP,
-  EngineID,
-  Status,
-  UnManaged,
-  Allow64BitCounters,
-  ObjectSubType,
-  SysObjectID,
-  MachineType,
-  VendorIcon,
-  SNMPVersion,
-  Community,
-  RediscoveryInterval,
-  PollInterval,
-  StatCollection,
-  Uri,
-  DisplayName
-  FROM Orion.Nodes where DisplayName='$ip' OR Caption='$ip'
-  };
+  my $query=$self->query_lookup($ip,$ip);
   my $result=$self->Query($query);
   $self->log_info("stopping");
   return $result;
 }
+
+=item * my $result=$self->getNodesByID($nodeid);
+
+Returns a Net::SolarWinds::Result object that contains a list of objects that matched that nodeid
+
+=cut
+
 sub getNodesByID {
   my ($self,$id)=@_;
   $self->log_info("starting");
-  my $query=qq{SELECT
-  NodeID,
-  IPAddress,
-  IPAddressGUID,
-  Caption,
-  DynamicIP,
-  EngineID,
-  Status,
-  UnManaged,
-  Allow64BitCounters,
-  ObjectSubType,
-  SysObjectID,
-  MachineType,
-  VendorIcon,
-  SNMPVersion,
-  Community,
-  RediscoveryInterval,
-  PollInterval,
-  StatCollection,
-  Uri
-  FROM Orion.Nodes where NodeId='$id'
-  };
+  my $query=$self->query_lookup($id);
   my $result=$self->Query($query);
 
   $self->log_info("stopping");
@@ -615,7 +622,8 @@ sub getNodeUri {
   my ($self,$node_id)=@_;
 
   $self->log_info("starting");
-  my $query="Select Uri from Orion.Nodes where NodeId=$node_id";
+  #my $query="Select Uri from Orion.Nodes where NodeId=$node_id";
+  my $query=$self->query_lookup($node_id);
 
   my $result=$self->Query($query);
 
@@ -634,7 +642,6 @@ sub getNodeUri {
   $self->log_info("stopping");
   return $self->RESULT_CLASS->new_true($result->get_data->{results}->[0]->{Uri});
 
-
 }
 
 =item * my $result=$self->deleteSwis($uri);
@@ -688,7 +695,7 @@ sub getApplicationTemplate {
 
   $self->log_info("starting");
   my $append=join ' OR ',map { sprintf q{Name='%s'},$_ } @names;
-  my $query=qq{SELECT ApplicationTemplateID, Created, CustomApplicationType, Description, DisplayName, HasImportedView, InstanceType, IsMockTemplate, LastModified, Name, Uri, ViewID, ViewXml FROM Orion.APM.ApplicationTemplate where $append};
+  my $query=$self->query_lookup($append);
  
   my $result=$self->Query($query);
   $self->log_info("stopping");
@@ -696,6 +703,7 @@ sub getApplicationTemplate {
 }
 
 =item * my $result=$self->addTemplateToNode($node_id,$template_id);
+
 =item * my $result=$self->addTemplateToNode($node_id,$template_id,$cred_id);
 
 Adds a monitoring template with the default credentals to the node.
@@ -741,7 +749,7 @@ when false it contains why it failed.
 sub getTemplatesOnNode {
   my ($self,$node_id)=@_;
     $self->log_info("starting");
-  my $query=qq{SELECT ApplicationID, ApplicationTemplateID, Description, DetailsUrl, DisplayName, HasCredentials, ID, InstanceType, Name, NodeID, Uri FROM Orion.APM.Application where nodeid=$node_id};
+  my $query=$self->query_lookup($node_id);
   my $result=$self->Query($query);
   unless($result) {
     $self->log_error("failed to getTemplatesOnNode");
@@ -878,7 +886,7 @@ sub getVolumeTypeMap {
   my ($self)=@_;
 
   $self->log_info("starting");
-  my $query=q{SELECT distinct VolumeType, VolumeTypeIcon, VolumeTypeID FROM Orion.Volumes};
+  my $query=$self->SWQL_getNodesByIp;
   my $result=$self->Query($query);
   unless($result) {
     $self->log_error("Failed to get getVolumeTypeMap error was: $result");
@@ -909,9 +917,7 @@ When false it contains why it failed.
 sub getEngines {
   my ($self)=@_;
 
-  my $result=$self->Query(q{
-SELECT AvgCPUUtil, BusinessLayerPort, CompanyName, CustomerID, Description, DisplayName, Elements, EngineID, EngineVersion, EvalDaysLeft, Evaluation, FailOverActive, FIPSModeEnabled, InstanceType, InterfacePollInterval, Interfaces, InterfaceStatPollInterval, IP, KeepAlive, LicensedElements, LicenseKey, MaxPollsPerSecond, MaxStatPollsPerSecond, MemoryUtil, MinutesSinceFailOverActive, MinutesSinceKeepAlive, MinutesSinceRestart, MinutesSinceStartTime, MinutesSinceSysLogKeepAlive, MinutesSinceTrapsKeepAlive, NodePollInterval, Nodes, NodeStatPollInterval, PackageName, Pollers, PollingCompletion, PrimaryServers, Restart, SerialNumber, ServerName, ServerType, ServicePack, StartTime, StatPollInterval, SysLogKeepAlive, TrapsKeepAlive, Uri, VolumePollInterval, Volumes, VolumeStatPollInterval, WindowsVersion FROM Orion.Engines
-  });
+  my $result=$self->Query($self->SWQL_getEngines);
 
   return $result;
 }
@@ -929,9 +935,7 @@ Notes: if no matching engines were found the result object can be true.. but wil
 sub getEngine {
   my ($self,$engine)=@_;
 
-  my $result=$self->Query(qq{
-SELECT AvgCPUUtil, BusinessLayerPort, CompanyName, CustomerID, Description, DisplayName, Elements, EngineID, EngineVersion, EvalDaysLeft, Evaluation, FailOverActive, FIPSModeEnabled, InstanceType, InterfacePollInterval, Interfaces, InterfaceStatPollInterval, IP, KeepAlive, LicensedElements, LicenseKey, MaxPollsPerSecond, MaxStatPollsPerSecond, MemoryUtil, MinutesSinceFailOverActive, MinutesSinceKeepAlive, MinutesSinceRestart, MinutesSinceStartTime, MinutesSinceSysLogKeepAlive, MinutesSinceTrapsKeepAlive, NodePollInterval, Nodes, NodeStatPollInterval, PackageName, Pollers, PollingCompletion, PrimaryServers, Restart, SerialNumber, ServerName, ServerType, ServicePack, StartTime, StatPollInterval, SysLogKeepAlive, TrapsKeepAlive, Uri, VolumePollInterval, Volumes, VolumeStatPollInterval, WindowsVersion FROM Orion.Engines where ServerName='$engine' or DisplayName='$engine'
-  });
+  my $result=$self->Query($self->query_lookup($engine,$engine));
 
   return $result unless $result;
 }
@@ -947,26 +951,7 @@ sub getVolumeMap {
   my ($self,$node_id)=@_;
 
   $self->log_info("starting");
-  my $query=qq{SELECT 
-  
-    VolumeIndex,
-    Caption,
-    VolumeDescription,
-    Status,
-    Type ,
-    Icon ,
-    VolumeSpaceAvailable,
-    VolumeSize,
-    VolumePercentUsed,
-    VolumeSpaceUsed,
-    VolumeTypeID,
-    PollInterval,
-    StatCollection,
-    RediscoveryInterval,
-    VolumeID,
-    NodeID
-  
-  FROM Orion.Volumes where NodeID=$node_id order by VolumeIndex desc};
+  my $query=$self->query_lookup($node_id);
   my $result=$self->Query($query);
   unless($result) {
     $self->log_error("Failed to get getVolumeMap for node_id: $node_id error was: $result");
@@ -1015,22 +1000,21 @@ Returns a Net::SolarWinds::Result Object that contains the nodes when true when
 
 sub GetNodePollers {
   my ($self,$node_id,$type)=@_;
+  $type='N' unless defined($type);
   $self->log_info("starting node_id: $node_id type: $type");
-  my $query=sprintf q{SELECT Description, DisplayName, Enabled, InstanceType, NetObject, NetObjectID, NetObjectType, PollerID, PollerType, Uri FROM Orion.Pollers where NetObjectID=%i},$node_id;
+  my $query=$self->query_lookup($node_id,$type);
   my $result=$self->Query($query);
 
-  $type='N' unless defined($type);
   unless($result) {
     $self->log_error("Failed to GetNodePollers on node_id: $node_id type: $type");
     $self->log_info("stopping");
     return $result;
   }
 
-
   my $list=$result->get_data->{results};
   my $pollers=[];
   foreach my $poller (@{$list}) {
-    my $result=$self->getSwisProps('swis://localhost/Orion/Orion.Pollers/PollerID='.$poller);
+    my $result=$self->getSwisProps('swis://localhost/Orion/Orion.Pollers/PollerID='.$poller->{PollerID});
     return unless $result;
     push @{$pollers},$result->get_data;
   }
