@@ -2,7 +2,7 @@ package Net::SolarWinds::REST::Batch;
 
 =head1 NAME
 
-Net::SolarWinds::REST::Batch
+Net::SolarWinds::REST::Batch - SolarWinds Batch Prcess class
 
 =head1 SYNOPSIS
 
